#include <Stepper.h>

// Please tune the following value if the clock gains or loses.
// Theoretically, standard of this value is 60000.
#define MILLIS_PER_MIN 60100 // milliseconcs per a minute

#define ROTATION_PERIOD_IN_MSEC 10000

// Motor and clock parameters
#define STEP 32 // steps of a single rotation of motor
#define REDUCTION 64 // reduction ratio in the motor
#define RATIO 5 // minutes per a rotation

#define LAP 262144  // value to avoid overflow of long int

Stepper stepper(STEP, 6, 4, 5, 7);

void setup() {
  stepper.setSpeed(50);
}

long calc_step(long msec) {
  return STEP * REDUCTION * msec / MILLIS_PER_MIN / RATIO;
}

void loop() {
  static long prev_msec, prev_pos;
  long msec, pos;

  msec = millis() % LAP;
  if(msec < prev_msec) { // Laps around
      msec += LAP; // hide lap around temporarily
    }
    
  if(msec >= prev_msec + ROTATION_PERIOD_IN_MSEC) { // do rotation
    pos = calc_step(msec);
    stepper.step(pos - prev_pos);

    if(msec >= LAP) { // Laps around
      msec %= LAP;
      pos = calc_step(msec);
    }
    prev_pos = pos;
    prev_msec = msec;
    
    // shutdown of stepping motor to suppress warming of motor and the earth
    delay(20);
    digitalWrite(4, LOW);
    digitalWrite(5, LOW);
    digitalWrite(6, LOW);
    digitalWrite(7, LOW);
  }
}
